import os

def checkNumber(value, field_name):
    try:
        float(value)
    except ValueError:
        raise Exception(str(field_name) + " is not a number")

    return value

def checkPositiveNumber(value, field_name):
    value = checkNumber(value, field_name)
    if (value < 0): raise Exception(field_name + " should be >= 0")

    return value

def checkStrictlyPositiveNumber(value, field_name):
    value = checkNumber(value, field_name)
    if (value <= 0): raise Exception(field_name + " should be > 0")

    return value

def checkAngle(value, field_name):
    value = checkNumber(value, field_name)
    if value < -360 or value > 360: raise Exception(field_name + " should be >= -360 and <= 360 deg")

    return value

def checkPositiveAngle(value, field_name):
    value = checkNumber(value, field_name)
    if value < 0 or value > 360: raise Exception(field_name + " should be >= 0 and <= 360 deg")

    return value

def checkStrictlyPositiveAngle(value, field_name):
    value = checkNumber(value, field_name)
    if value <= 0 or value >= 360: raise Exception(field_name + " should be > 0 and < 360 deg")

    return value

def checkEmptyString(string, field_name):
    if string is None: raise Exception(field_name + " should not be an empty string")
    if string.strip() == "": raise Exception(field_name + " should not be an empty string")

    return string

def checkGreaterThan(number1, number2, field_name1, field_name_2):
    if number1 <= number2: raise Exception(field_name1 + " should be greater than " + field_name_2)

def checkGreaterOrEqualThan(number1, number2, field_name1, field_name_2):
    if number1 < number2: raise Exception(field_name1 + " should be greater or equal than " + field_name_2)

def checkLessThan(number1, number2, field_name1, field_name_2):
    if number1 >= number2: raise Exception(field_name1 + " should be less than " + field_name_2)

def checkLessOrEqualThan(number1, number2, field_name1, field_name_2):
    if number1 > number2: raise Exception(field_name1 + " should be less or equal than " + field_name_2)

def checkEqualTo(number1, number2, field_name1, field_name_2):
    if number1 != number2: raise Exception(field_name1 + " should be equal to " + field_name_2)

def checkFileName(fileName):
    if fileName is None: raise Exception("File name is Empty")
    if fileName.strip() == "": raise Exception("File name is Empty")

    if os.path.isabs(fileName):
        filePath = fileName
    else:
        if fileName.startswith('/'):
            filePath =  os.getcwd() + fileName
        else:
            filePath = os.getcwd() + '/' + fileName

    return filePath

def checkDir(fileName):
    filePath = checkFileName(fileName)

    container_dir = os.path.dirname(filePath)

    if not os.path.exists(container_dir):
        raise Exception("Directory " + container_dir + " not existing")

    return filePath

def checkFile(fileName):
    filePath = checkDir(fileName)

    if not os.path.exists(filePath):
        raise Exception("File " + fileName + " not existing")

    return filePath



import numpy

from PyQt5.QtCore import QRect
from PyQt5.QtWidgets import QApplication, QMessageBox

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from silx.gui import qt
import silx.gui.hdf5
from silx.gui.data.DataViewerFrame import DataViewerFrame

class Hdf5TreeViewWidget(qt.QWidget):
    def __init__(self, file_names=None):
        qt.QWidget.__init__(self)

        self.__treeview = silx.gui.hdf5.Hdf5TreeView(self)
        self.__text = qt.QTextEdit(self)
        self.__dataViewer = DataViewerFrame(self)

        vSplitter = qt.QSplitter(qt.Qt.Vertical)
        vSplitter.addWidget(self.__dataViewer)
        vSplitter.addWidget(self.__text)
        vSplitter.setSizes([10, 0])

        splitter = qt.QSplitter(self)
        splitter.addWidget(self.__treeview)
        splitter.addWidget(vSplitter)
        splitter.setStretchFactor(1, 1)

        layout = qt.QVBoxLayout()
        layout.addWidget(splitter)
        layout.setStretchFactor(splitter, 1)
        self.setLayout(layout)

        # append all files to the tree
        if not file_names is None:
            for file_name in file_names:
                self.__treeview.findHdf5TreeModel().appendFile(file_name)

        self.__treeview.activated.connect(self.displayData)

    def displayData(self):
        """Called to update the dataviewer with the selected data.
        """
        selected = list(self.__treeview.selectedH5Nodes())
        if len(selected) == 1:
            # Update the viewer for a single selection
            data = selected[0]
            self.__dataViewer.setData(data)

    def load_file(self, filename):
        self.__treeview.findHdf5TreeModel().insertFile(filename)

    def set_text(self, text):
        self.__text.setText(text)

    def __hdf5ComboChanged(self, index):
        function = self.__hdf5Combo.itemData(index)
        self.__createHdf5Button.setCallable(function)

    def __edfComboChanged(self, index):
        function = self.__edfCombo.itemData(index)
        self.__createEdfButton.setCallable(function)


class OWHDF5FileReader(OWWidget):
    name = "HDF5 File Reader"
    id = "hdf5_file_reader"
    description = "HDF5 File Reader"
    icon = "icons/hdf5.png"
    author = "Luca Rebuffi"
    maintainer_email = "lrebuffi@anl.gov"
    priority = 3
    category = ""
    keywords = ["hdf5_file_reader"]

    want_main_area = 1
    want_control_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 618

    hdf5_file_name = Setting('file.hdf5')

    def __init__(self):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        gui.separator(self.controlArea)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Load HDF5 file", callback=self.load_file)
        button.setFixedHeight(45)

        input_box_l = oasysgui.widgetBox(self.controlArea, "Input", addSpace=True, orientation="horizontal", height=self.TABS_AREA_HEIGHT)

        self.le_hdf5_file_name = oasysgui.lineEdit(input_box_l, self, "hdf5_file_name", "HDF5 File Name",
                                                        labelWidth=120, valueType=str, orientation="horizontal")

        gui.button(input_box_l, self, "...", callback=self.selectPlotXYFile)

        self.tree_view = Hdf5TreeViewWidget()

        self.mainArea.layout().addWidget(self.tree_view)

        gui.rubber(self.mainArea)


    def load_file(self):
        try:
            hdf5_file_name = congruence.checkDir(self.hdf5_file_name)

            self.tree_view.load_file(hdf5_file_name)
            self.tree_view.set_text("Loaded File: " + hdf5_file_name)

        except Exception as exception:
            QMessageBox.critical(self, "Error", exception.args[0], QMessageBox.Ok)

            if self.IS_DEVELOP: raise exception

    def selectPlotXYFile(self):
        self.le_hdf5_file_name.setText(oasysgui.selectFileFromDialog(self, self.hdf5_file_name, "Select Input File", file_extension_filter="HDF5 Files (*.hdf5 *.h5 *.hdf)"))





__author__ = 'labx'

"""
=========
Tools
=========

Widgets for common tools.

"""

# Category description for the widget registry

NAME = "Tools"

DESCRIPTION = "Widgets for Common Tools."

BACKGROUND = "#FAAC58"

ICON = "icons/tools.png"

PRIORITY = 1




# THIS FILE IS GENERATED FROM OASYS SETUP.PY
short_version = '1.1.31'
version = '1.1.31'
full_version = '1.1.31'
git_revision = '14d2224b9b2948a18b73968940155038ccf1b59e'
release = True

if not release:
    version = full_version
    short_version += ".dev"



import os, sys

import numpy
from PyQt5.QtCore import QRect, Qt
from PyQt5.QtWidgets import QApplication, QMessageBox, QLabel, QSizePolicy
from PyQt5.QtGui import QTextCursor, QFont, QPalette, QColor, QPixmap

from matplotlib import cm
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure

from orangewidget import gui, widget
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from oasys.util.oasys_objects import OasysSurfaceData

import oasys.util.oasys_util as OU

try:
    from mpl_toolkits.mplot3d import Axes3D  # necessario per caricare i plot 3D
except:
    pass

class OWSurfaceFileReader(OWWidget):
    name = "Surface File Reader"
    id = "surface_file_reader"
    description = "Surface File Reader"
    icon = "icons/surface.png"
    author = "Luca Rebuffi"
    maintainer_email = "lrebuffi@anl.gov"
    priority = 2
    category = ""
    keywords = ["surface_file_reader"]

    outputs = [{"name": "Surface Data",
                "type": OasysSurfaceData,
                "doc": "Surface Data",
                "id": "Surface Data"}]

    want_main_area = 1
    want_control_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 618

    xx = None
    yy = None
    zz = None

    surface_file_name = Setting('surface.hdf5')

    def __init__(self):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        gui.separator(self.controlArea)

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Render Surface", callback=self.render_surface)
        button.setFixedHeight(45)

        input_box_l = oasysgui.widgetBox(self.controlArea, "Input", addSpace=True, orientation="horizontal", height=self.TABS_AREA_HEIGHT)

        self.le_surface_file_name = oasysgui.lineEdit(input_box_l, self, "surface_file_name", "Surface File Name",
                                                        labelWidth=120, valueType=str, orientation="horizontal")

        gui.button(input_box_l, self, "...", callback=self.selectSurfaceFile)


        self.figure = Figure(figsize=(600, 600))
        self.figure.patch.set_facecolor('white')

        self.axis = self.figure.add_subplot(111, projection='3d')

        self.axis.set_zlabel("Z [m]")

        self.figure_canvas = FigureCanvasQTAgg(self.figure)

        self.mainArea.layout().addWidget(self.figure_canvas)

        gui.rubber(self.mainArea)


    def render_surface(self):
        try:
            self.surface_file_name = congruence.checkDir(self.surface_file_name)

            #### LENGTH

            xx, yy, zz = OU.read_surface_file(self.surface_file_name)

            self.xx = xx
            self.yy = yy
            self.zz = zz

            self.axis.clear()

            x_to_plot, y_to_plot = numpy.meshgrid(self.xx, self.yy)

            self.axis.plot_surface(x_to_plot, y_to_plot, zz,
                                   rstride=1, cstride=1, cmap=cm.autumn, linewidth=0.5, antialiased=True)

            self.axis.set_xlabel("X [m]")
            self.axis.set_ylabel("Y [m]")
            self.axis.set_zlabel("Z [m]")
            self.axis.mouse_init()

            self.figure_canvas.draw()

            self.send("Surface Data", OasysSurfaceData(xx=self.xx, yy=self.yy, zz=self.zz, surface_data_file=self.surface_file_name))

        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                 exception.args[0],
                                 QMessageBox.Ok)

            if self.IS_DEVELOP: raise exception

    def selectSurfaceFile(self):
        self.le_surface_file_name.setText(oasysgui.selectFileFromDialog(self, self.surface_file_name, "Select Input File", file_extension_filter="HDF5 Files (*.hdf5 *.h5 *.hdf)"))





import logging

import pkg_resources

from PyQt5.QtGui import QPixmap, QFont, QFontMetrics, QColor, QPainter, QIcon
from PyQt5.QtCore import Qt, QCoreApplication, QPoint, QRect

from orangewidget.canvas import config as owconfig
from orangecanvas import config

from . import discovery, widgetsscheme


WIDGETS_ENTRY = "oasys.widgets"
MENU_ENTRY = "oasys.menus"

#: Parameters for searching add-on packages in PyPi using xmlrpc api.
ADDON_PYPI_SEARCH_SPEC = {"keywords": "oasys1", "owner" : "lucarebuffi"}
#: Entry points by which add-ons register with pkg_resources.
ADDONS_ENTRY = "oasys.addons"

# Add a default for our extra default-working-dir setting.
config.spec += [
    config.config_slot("output/default-working-dir", str, "",
                       "Default working directory"),
    config.config_slot("oasys/addon-update-check-period", int, 1,
                       "Check for updates every (in days)")
]


class oasysconf(owconfig.orangeconfig):
    OrganizationDomain = ""
    ApplicationName = "OASYS1"
    ApplicationVersion = "1.1"

    @staticmethod
    def splash_screen():
        path = pkg_resources.resource_filename(
            __name__, "icons/oasys-splash-screen.png")
        pm = QPixmap(path)

        version = QCoreApplication.applicationVersion()
        size = 21 if len(version) < 5 else 16
        font = QFont("Helvetica")
        font.setPixelSize(size)
        font.setBold(True)
        font.setItalic(True)
        font.setLetterSpacing(QFont.AbsoluteSpacing, 2)
        metrics = QFontMetrics(font)
        br = metrics.boundingRect(version).adjusted(-5, 0, 5, 0)
        br.moveCenter(QPoint(436, 224))

        p = QPainter(pm)
        p.setRenderHint(QPainter.Antialiasing)
        p.setRenderHint(QPainter.TextAntialiasing)
        p.setFont(font)
        p.setPen(QColor("#231F20"))
        p.drawText(br, Qt.AlignCenter, version)
        p.end()
        return pm, QRect(88, 193, 200, 20)

    @staticmethod
    def application_icon():
        """
        Return the main application icon.
        """
        path = pkg_resources.resource_filename(
            __name__, "icons/oasys.png"
        )
        return QIcon(path)

    @staticmethod
    def widgets_entry_points():
        return pkg_resources.iter_entry_points(WIDGETS_ENTRY)

    @staticmethod
    def addon_entry_points():
        return pkg_resources.iter_entry_points(ADDONS_ENTRY)

    @staticmethod
    def addon_pypi_search_spec():
        return dict(ADDON_PYPI_SEARCH_SPEC)

    @staticmethod
    def tutorials_entry_points():
        return pkg_resources.iter_entry_points("oasys.tutorials")

    workflow_constructor = widgetsscheme.OASYSWidgetsScheme


def omenus():
    """
    Return an iterator of oasys.menu.OMenu instances registered
    by 'orange.menu' pkg_resources entry point.
    """
    log = logging.getLogger(__name__)
    for ep in pkg_resources.iter_entry_points(MENU_ENTRY):
        try:
            menu = ep.load()
        except pkg_resources.ResolutionError:
            log.info("Error loading a '%s' entry point.", MENU_ENTRY,
                     exc_info=True)
        except Exception:
            log.exception("Error loading a '%s' entry point.",
                          MENU_ENTRY)
        else:
            if "MENU" in menu.__dict__:
                yield from discovery.omenus_from_package(menu)


def menu_registry():
    """
    Return the the OASYS extension menu registry.
    """
    return discovery.MenuRegistry(list(omenus()))
